"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT = exports.COMPOSE_NULLISH = exports.COMPOSE = void 0;
const pick_1 = __importDefault(require("@cookiex/dot-object/dist/pick"));
const set_1 = __importDefault(require("@cookiex/dot-object/dist/set"));
const combineReducers = (map, nullish = false) => {
    const reducers = Object.entries(map);
    return (state, action) => {
        if (nullish && !state)
            return state;
        let newerState = state;
        for (const [key, reducer] of reducers) {
            const newer = reducer(state[key], action);
            if (state[key] !== newer)
                newerState = Object.assign(Object.assign({}, newerState), { [key]: newer });
        }
        return newerState;
    };
};
const extractKeys = (data) => [
    ...Object.getOwnPropertyNames(data),
    ...Object.getOwnPropertySymbols(data),
];
const createReducer = (config, initialState) => {
    const keys = extractKeys(config).filter(privateFilter);
    const composeIsNullish = exports.COMPOSE_NULLISH in config;
    const hasCompose = composeIsNullish || exports.COMPOSE in config;
    const composeKey = exports.COMPOSE in config ? exports.COMPOSE : exports.COMPOSE_NULLISH;
    const sub = hasCompose && [
        ...Object.getOwnPropertyNames(config[composeKey]),
        ...Object.getOwnPropertySymbols(config[composeKey])
    ].reduce((reducers, key) => (Object.assign(Object.assign({}, reducers), { [key]: createReducer(config[composeKey][key], initialState[key]) })), {});
    const combine = exports.COMPOSE in config && combineReducers(sub, composeIsNullish);
    const reducer = (state = initialState, action) => {
        const key = keys.find(key => key === action.type);
        if (action.target) {
            const target = pick_1.default(action.target, reducer);
            if (target)
                return set_1.default(action.target, state, target(pick_1.default(action.target, state), action));
        }
        if (key in config)
            return config[key](state, action.data);
        if (exports.COMPOSE in config)
            return combine(state, action);
        if (exports.DEFAULT in config)
            return config[exports.DEFAULT](state, action);
        return (state !== null && state !== void 0 ? state : {});
    };
    Object.assign(reducer, sub);
    return reducer;
};
exports.COMPOSE = Symbol('store-reducer-compose');
exports.COMPOSE_NULLISH = Symbol('store-reducer-compose-nullish');
exports.DEFAULT = Symbol('store-reducer-default');
const privateKeys = [exports.COMPOSE, exports.DEFAULT];
const privateFilter = (key) => !privateKeys.includes(key);
exports.default = createReducer;
